package com.tzld.piaoquan.risk.control.service.impl;

import com.alibaba.fastjson.JSON;
import com.tzld.piaoquan.risk.control.config.QywxConfig;
import com.tzld.piaoquan.risk.control.dao.mapper.WorkWechatRoomInfoMapper;
import com.tzld.piaoquan.risk.control.model.po.WorkWechatRoomInfo;
import com.tzld.piaoquan.risk.control.model.po.WorkWechatRoomInfoExample;
import com.tzld.piaoquan.risk.control.model.qywx.RoomListResponse;
import com.tzld.piaoquan.risk.control.model.qywx.UserListFromAdPlatformModel;
import com.tzld.piaoquan.risk.control.util.HttpClientUtil;
import com.tzld.piaoquan.risk.control.util.HttpPoolClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class RiskRuleConfigService {
    @Autowired
    private WorkWechatServiceImpl workWechatServiceImpl;
    private static final int USER_COUNT_THRESHOLD = 5; //生效人数阈值
    private static final int USER_COUNT_THRESHOLD_FOR_ANOMALY_USER_RATIO = 30; //大于30人才触发是否关闭判断
    private static final float ANOMALY_USER_RATIO = 0.1f; //异常用户比例20%
    @Autowired
    private WorkWechatRoomInfoMapper mapper;

    @Autowired
    private QywxConfig config;
    @Value("${qywx.corpid}")
    private long corpid;
    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(10000, 10000, 2000, 5000, 5, 10000);
    @Value("${getUserList.url:https://api.piaoquantv.com/ad/platform/wechat/group/groupUserDetailData}")
    private String getUserList;
    Map<String, List<RoomListResponse.RoomInfo>> getKickOpenedRoom(Map<String, List<RoomListResponse.RoomInfo>> tobeOperate) {
        Map<String, List<RoomListResponse.RoomInfo>> filteredMap = new HashMap<>();
        for (Map.Entry<String, List<RoomListResponse.RoomInfo>> entry : tobeOperate.entrySet()) {
            List<RoomListResponse.RoomInfo> filteredRooms = new ArrayList<>();

            for (RoomListResponse.RoomInfo room : entry.getValue()) {
                if (workWechatServiceImpl.checkAutoRemoveUserStatusIsOpen(room.getRoomId())) {
                    filteredRooms.add(room);
                }
            }

            if (!filteredRooms.isEmpty()) {
                filteredMap.put(entry.getKey(), filteredRooms);
            }
        }

        return filteredMap;
    }



    //防止误踢，只有满足存在五个正常群人数才会生效
    public FixedRuleResult getFixedRulesForThisChat(String chatId, String roomId) {
        FixedRuleResult ruleResult = new FixedRuleResult();
        WorkWechatRoomInfoExample example = new WorkWechatRoomInfoExample();
        example.createCriteria().andRoomIdEqualTo(roomId);
        List<WorkWechatRoomInfo> examples = mapper.selectByExample(example);
        if (examples == null || examples.isEmpty()) {
            log.error("getFixedRulesForThisChat, no room found for roomId: {}", roomId);
            return ruleResult;
        }
        WorkWechatRoomInfo roomInfo = examples.get(0);
        long roomUpdateTime =  roomInfo.getUpdateTime().getTime()/1000;
        log.info("getFixedRulesForThisChat, roomId: {}, roomUpdateTime: {}", roomId, roomUpdateTime);
        List<UserListFromAdPlatformModel.ChatMember> memberList = getUserListFromAdPlatform(chatId);
        List<UserListFromAdPlatformModel.ChatMember> distinctMembers = memberList.stream()
                .collect(Collectors.toMap(
                        UserListFromAdPlatformModel.ChatMember::getExternalId,  // key: externalId
                        member -> member,                                       // value: ChatMember 对象
                        (existing, replacement) ->                              // 合并策略：取 joinTime 更大的那条
                                existing.getJoinTime() > replacement.getJoinTime() ? existing : replacement
                ))
                .values()  // 获取去重后的成员列表
                .stream()
                .collect(Collectors.toList());
        if (distinctMembers == null || distinctMembers.isEmpty()) {
            log.error("checkIfOpenedForFixedRules, memberList is empty for chatId: {}", chatId);
            return ruleResult;
        } else {
            // 打印所有成员的joinTime和roomUpdateTime用于调试
            distinctMembers.forEach(member -> {
                if (member.getJoinTime() != null) {
                    log.info(
                            "Member ID: " + member.getNickName() +
                                    ", joinTime: " + member.getJoinTime() +
                                    ", roomUpdateTime: " + roomUpdateTime +
                                    ", isAfter: " + (member.getJoinTime() > roomUpdateTime)
                    );
                } else {
                    System.out.println("Member ID: " + member.getNickName() + ", joinTime: null");
                }
            });
            //开关开启后的加入的数量
            int size = (int) distinctMembers.stream()
                    .filter(member -> member.getJoinTime() != null && member.getJoinTime() > roomUpdateTime)
                    .count();;//群总数，包括异常和正常
            //memberList中isAbnormal是0代表正常用户,判断joinTime是否大于updateTime
            int normalSize = (int) distinctMembers.stream()
                    .filter(member -> member.getIsAbnormal() == 0).filter(
                            member -> member.getJoinTime() != null && member.getJoinTime() > roomUpdateTime)
                    .count();
            ruleResult.normalCount = normalSize;
            ruleResult.abnormalCount = size - normalSize;
            log.info("checkIfOpenedForFixedRules, chatId: {}, size: {}, normalSize: {}，total size:{}", chatId, size, normalSize);

            if(normalSize >= USER_COUNT_THRESHOLD) {//大于等于3才生效
                log.info("checkIfOpenedForFixedRules, size: {}, normalSize: {}", size, normalSize);
                //如果群成员大于30人，才判断异常用户比例
                ruleResult.isNormalCountEnough = true;
                if (size >= USER_COUNT_THRESHOLD_FOR_ANOMALY_USER_RATIO) {
                    //计算异常用户比例
                    float anomalyUserRatio = (float) (size - normalSize) / size;
                    log.info("checkIfOpenedForFixedRules, chatId: {}, anomalyUserRatio: {}", chatId, anomalyUserRatio);
                    if (anomalyUserRatio >= ANOMALY_USER_RATIO) {
                        log.info("checkIfOpenedForFixedRules, chatId: {}, anomalyUserRatio is high enough, opening rules", chatId);
                        ruleResult.isAbnormalRatioHighEnough = true;
                    } else {
                        log.info("checkIfOpenedForFixedRules, chatId: {}, anomalyUserRatio is not high enough, not opening rules", chatId);
                    }
                }
                return ruleResult;
            } else {
                log.info("checkIfOpenedForFixedRules, chatId: {}, size: {}, normalSize: {}, not enough normal users", chatId, size, normalSize);
                return ruleResult;
            }
        }
    }



    private List<UserListFromAdPlatformModel.ChatMember> getUserListFromAdPlatform(String chatId) {
        String url = String.format("%s?chatId=%s", getUserList, chatId);
        List<UserListFromAdPlatformModel.ChatMember> result = Collections.emptyList();
        log.info("getUserListFromAdPlatform, url: {}", url);
        Optional<String> response = httpPoolClientDefault.get(url);
        if (!response.isPresent()) {
            log.error("getUserListFromAdPlatform, response is empty for chatId: {}", chatId);
        } else {
            log.info("getUserListFromAdPlatform, response: {}", response.get());
            UserListFromAdPlatformModel model =  JSON.parseObject(response.get(), UserListFromAdPlatformModel.class);
            if (model != null && model.getData() != null) {
                result = model.getData();
            } else {
                log.error("getUserListFromAdPlatform, model is null or data is empty for chatId: {}", chatId);
                return new ArrayList<>();
            }
        }
        return result;
    }

    public static class FixedRuleResult {
        boolean isNormalCountEnough = false;
        int normalCount = 0;
        boolean isAbnormalRatioHighEnough = false;
        float abnormalUserRatio = 0.0f;
        int abnormalCount = 0;

        @Override
        public String toString() {
            return "FixedRuleResult{" +
                    "isNormalCountEnough=" + isNormalCountEnough +
                    ", normalCount=" + normalCount +
                    ", isAbnormalRatioHighEnough=" + isAbnormalRatioHighEnough +
                    ", abnormalUserRatio=" + abnormalUserRatio +
                    ", abnormalCount=" + abnormalCount +
                    '}';
        }
    }
}
