package com.tzld.piaoquan.risk.control.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.tzld.piaoquan.risk.control.config.QywxConfig;
import com.tzld.piaoquan.risk.control.model.po.UserBase;
import com.tzld.piaoquan.risk.control.model.qywx.*;
import com.tzld.piaoquan.risk.control.util.HttpClientUtil;
import com.tzld.piaoquan.risk.control.util.HttpPoolClient;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

@Slf4j
@Service
public class RiskUserHandleService {
    @Autowired
    private QywxConfig qywxConfig; // 注入配置类
    @Autowired
    private QywxUserDataService qwUserService;
    @Autowired
    private RiskUserOperateService riskUserOperateService;
    @Autowired
    private RiskRuleConfigService riskRuleConfigService;

    @Value("${qywx.corpid}")
    private String corpid;
    @Value("${qywx.scorpid}")
    private String scorpid;
    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(10000, 10000, 2000, 5000, 5, 10000);
    private static final Logger LOGGER = LoggerFactory.getLogger(RiskUserHandleService.class);


    public int handleRiskUser(RiskUserInfo riskUserInfo){
        log.info("handleRiskUser, riskUserInfo: {}", riskUserInfo);
        //根据名称找到人
        List<UserBase> staffList = findStaffByName(riskUserInfo);
        if (staffList.isEmpty()) return 0;
        //根据群名匹配到：人-群:哪个员工哪个群
        Map<String, List<RoomListResponse.RoomInfo>> toBeOperate = matchUserAndRoom(staffList, riskUserInfo);
        if (toBeOperate.isEmpty()) return 0;
        log.info("handleRiskUser, toBeOperate: {}", toBeOperate);
        toBeOperate = riskRuleConfigService.getKickOpenedRoom(toBeOperate);
        //找到待踢的人,找到终止
        for (Map.Entry<String, List<RoomListResponse.RoomInfo>> entry : toBeOperate.entrySet()) {
            String staffVid = entry.getKey(); // Key: staff vid内部员工
            UserBase staff = findStaffByVid(staffList, staffVid);
            List<RoomListResponse.RoomInfo> roomInfoList = entry.getValue(); // Value: List of RoomInfo
            // Process each room in the list
            long externalVid =  externalId2Vid(staff, riskUserInfo);
            if (externalVid == -1) {
                log.info("handleRiskUser, externalId2Vid error, staff: {}, riskUserInfo: {}", staff, riskUserInfo);
                continue;
            }
            //这里的room都是已经打开的
            for (RoomListResponse.RoomInfo roomInfo : roomInfoList) {
                if (riskUserOperateService.isInRoom(staff,Long.parseLong(roomInfo.getRoomId()),externalVid)){//risk用户在这个群内，且开关打开
                    //固定规则校验，chatId转化到roomId
                    long roomId = chatIdToRoomId(staff.getUuid(), riskUserInfo.getChatId());
                    if ((roomId + "").equals(roomInfo.getRoomId())) {//匹配群
                        log.info("handleRiskUser, chatIdToRoomId , staff: {}, riskUserInfo: {}", staff, riskUserInfo);
                        RiskRuleConfigService.FixedRuleResult ruleResult = riskRuleConfigService.getFixedRulesForThisChat(riskUserInfo.getChatId(),roomId +"");
                        log.info("handleRiskUser, ruleResult: {}, roomInfo: {}", ruleResult, roomInfo);
                        if(ruleResult.isNormalCountEnough && !ruleResult.isAbnormalRatioHighEnough) {//正常进入用户满足需求，且异常率符合阈值
                            boolean success = riskUserOperateService.kick(staff,Long.parseLong(roomInfo.getRoomId()),externalVid);
                            if (success) {
                                log.info("handleRiskUser to be kick user, vid: {},name: {},chatId: {},chatName:{} ", externalVid,riskUserInfo.getExternalNickname(),riskUserInfo.getChatId(),riskUserInfo.getGroupName());
                                return 1;
                            }
                        } else {
                            if(ruleResult.isAbnormalRatioHighEnough) {//关闭群开关，写入展示提示信息字段
                                riskUserOperateService.forceCloseRoomSwitch(roomId +"");
                            }
                            log.info("handleRiskUser, ruleResult not enough, isNormalCountEnough: {}, isAnomalyUserRatioEnough: {}", ruleResult.isNormalCountEnough, ruleResult.isAbnormalRatioHighEnough);
                        }
                    }

                }

            }
        }
        return 0;
    }

    private UserBase findStaffByVid(List<UserBase> staffList, String vid) {
        for (UserBase userBase : staffList) {
            if (userBase.getVid().equals(vid)) {
                return userBase;
            }
        }
        return null;
    }

    private Map<String, List<RoomListResponse.RoomInfo>> matchUserAndRoom(List<UserBase> staffList, RiskUserInfo userInfo) {
        Map<String, List<RoomListResponse.RoomInfo>> toBeOperate = new HashMap<>();
        if (staffList != null && !staffList.isEmpty()) {
            log.info("matchUserAndRoom risk info", userInfo);
            for (UserBase userBase : staffList) {
                //获取匹配到每个人的群信息
                List<RoomListResponse.RoomInfo> chatRoomList = getChatList(userBase);
                if (chatRoomList != null && !chatRoomList.isEmpty()) {
                    log.info("handleRiskUserList chat room hit, userInfo: {}", userBase);
                    for (RoomListResponse.RoomInfo roomInfo : chatRoomList) {
                        List<RoomListResponse.RoomInfo> matched = findRoomByChatRoomName(chatRoomList, userInfo);
                        if (matched != null && !matched.isEmpty()) {
                            log.info("handleRiskUserList hits, userInfo: {}", userBase);
                            toBeOperate.put(userBase.getVid(), matched);
                        } else {
                            log.info("handleRiskUserList, roomInfo: {}", roomInfo);
                        }
                    }
                }
            }
        }
        return toBeOperate;
    }

    //每个人有几个匹配到的群
    public List<RoomListResponse.RoomInfo> findRoomByChatRoomName(List<RoomListResponse.RoomInfo> roomInfoList, RiskUserInfo riskInfo) {
        List<RoomListResponse.RoomInfo> matchedRoomList = new ArrayList<>();
        for (RoomListResponse.RoomInfo roomInfo : roomInfoList) {
            if (riskInfo != null && riskInfo.getGroupName().equals(roomInfo.getNickname())) {
                matchedRoomList.add(roomInfo);
            }
        }
        return matchedRoomList;
    }

    public List<RoomListResponse.RoomInfo> getChatList(UserBase userBase) {
        List<RoomListResponse.RoomInfo> chatRoomList = new ArrayList<>();
        Map<String, Object> requestBody = new HashMap<>();
        requestBody.put("uuid", userBase.getUuid());
        requestBody.put("limit", 100);
        requestBody.put("star_index", 0);
        log.info("getChatList, userBase: {}", userBase);
        String params = JSON.toJSONString(requestBody);
        Optional<String> response = httpPoolClientDefault.postJson(qywxConfig.getDomain() + qywxConfig.getPath("get-chatList"), params);
        QwCommonResModel<RoomListResponse> roomList = null;
        log.info("getChatList, params: {}, response: {}", params, response);
        if (response.isPresent()) {
            roomList = QwCommonResModel.parseResponse(response.get(), RoomListResponse.class);
        }
        if (roomList != null && roomList.getErrcode() == 0) {
            if (roomList.getData().getRoomList() != null && !roomList.getData().getRoomList().isEmpty()) {
                chatRoomList = roomList.getData().getRoomList();
                log.info("handleRiskUserList hits, userInfo: {}", userBase);
            } else {
                log.info("handleRiskUserList not hits, userInfo: {}", userBase);
            }
            log.info("handleRiskUserList, roomList size: {}", roomList.getData().getRoomList().size());
        }
        return chatRoomList;
    }

    public List<UserBase> findStaffByName(RiskUserInfo userInfo) {
        List<UserBase> userBaseList = new ArrayList<>();
        if (userInfo != null ) {
            List<RiskUserInfo.Admin> adminList = userInfo.getAdminList();
//            List<UserBase> staffList = qwUserService.getUserByNickName(userInfo.getExternalNickname());
            if (adminList != null && !adminList.isEmpty()) {
                for (RiskUserInfo.Admin admin : adminList) {
                    String name = admin.getNickname();
                    if (name != null && !name.isEmpty()) {
                        log.info("findStaffByName handleRiskUserList hits, userInfo: {}", name);
                        List<UserBase> matched = qwUserService.getUserByRealName(name);
                        if (matched != null && !matched.isEmpty()) {
                            log.info("findStaffByName find {} ",matched.get(0));
                            userBaseList.addAll(matched);
                        } else {
//                            log.info("handleRiskUserList not hits, userInfo: {}", name);
                        }
                    }
                }
            }
        }
        if (!userBaseList.isEmpty()) {
            log.info("handleRiskUserList hits, userInfo: {}", userInfo);
        }
        return userBaseList;
    }

    private long externalId2Vid(UserBase staff,RiskUserInfo riskinfo) {
        log.info("externalId2Vid, staff: {}, riskinfo: {}", staff, riskinfo);
        Map<String, Object> requestBody = new HashMap<>();
        requestBody.put("uuid", staff.getUuid());
        requestBody.put("corpid", corpid);
        requestBody.put("scorpid", scorpid);
        requestBody.put("openid",Arrays.asList(riskinfo.getExternalId()));
        String params = JSON.toJSONString(requestBody);
        log.info("externalId2Vid, params: {}", params);
        String url = qywxConfig.getDomain() + qywxConfig.getPath("userId2Vid");
        Optional<String> response = httpPoolClientDefault.postJson(url, params);
        log.info("externalId2Vid, response: {}", response);
        if (response.isPresent()) {
            QwCommonResModel<List<OpenUserInfo>> result = JSON.parseObject(
                    response.get(),
                    new TypeReference<QwCommonResModel<List<OpenUserInfo>>>() {}.getType()
            );
            if (result.getErrcode() == 0 && result.getData() != null && !result.getData().isEmpty()) {
                return result.getData().get(0).getUserId();
            } else {
                log.error("externalId2Vid error, response: {}", response.get());
            }
        } else {
            log.error("externalId2Vid error, no response");
        }
        return -1;
    }

    public long chatIdToRoomId(String uuid,String chatId) {
        long room_id = -1;
        String url = qywxConfig.getDomain() + qywxConfig.getPath("chatIdToRoomId");
        Map<String, Object> requestBody = new HashMap<>();
        requestBody.put("uuid", uuid);
        requestBody.put("corpid", corpid);
        requestBody.put("chatid", chatId);
        Optional<String> result = httpPoolClientDefault.postJson(url,JSON.toJSONString(requestBody));
        log.info("chatIdToRoomId, url: {}, requestBody: {}, result: {}", url, requestBody, result);
        if (result.isPresent()) {
            try {
                room_id = JSON.parseObject(result.get()).getJSONObject("data").getLong("room_id");
            } catch (Exception e) {
                log.error("findRoomAndClosed, error parsing response: {}", result.get(), e);
            }
        }
        log.info("chatIdToRoomId, room_id: {}", room_id);
        return room_id;
    }

}
