package com.tzld.piaoquan.risk.control.service.impl;

import com.alibaba.fastjson.JSON;
import com.tzld.piaoquan.risk.control.config.QywxConfig;
import com.tzld.piaoquan.risk.control.dao.mapper.WorkWechatRoomInfoMapper;
import com.tzld.piaoquan.risk.control.model.po.UserBase;
import com.tzld.piaoquan.risk.control.model.po.WorkWechatRoomInfo;
import com.tzld.piaoquan.risk.control.model.po.WorkWechatRoomInfoExample;
import com.tzld.piaoquan.risk.control.model.qywx.QwCommonResModel;
import com.tzld.piaoquan.risk.control.model.qywx.RiskUserInfo;
import com.tzld.piaoquan.risk.control.model.qywx.RoomListResponse;
import com.tzld.piaoquan.risk.control.model.qywx.RoomMemberListInfo;
import com.tzld.piaoquan.risk.control.util.HttpClientUtil;
import com.tzld.piaoquan.risk.control.util.HttpPoolClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

@Slf4j
@Service
public class RiskUserOperateService {
    @Autowired
    private QywxConfig qywxConfig; // 注入配置类
    @Autowired
    private WorkWechatRoomInfoMapper workWechatRoomInfoMapper;
    @Value("${qywx.corpid}")
    private long corpid;
    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(10000, 10000, 2000, 5000, 5, 10000);
    private static final Logger LOGGER = LoggerFactory.getLogger(RiskUserHandleService.class);

    public boolean checkAndKickExternalUser(UserBase staff,RiskUserInfo riskUserInfo,long vid,long roomId) {
        LOGGER.info("checkAndKickExternalUser, staff: {}, riskUserInfo: {}, vid: {}, roomId: {}", staff, riskUserInfo, vid, roomId);
        boolean inRoom = isInRoom(staff, roomId, vid);
        if (!inRoom) {
            LOGGER.info("User {} is not in room {}", vid, roomId);
            return false;
        }
        LOGGER.info("User {} is in room {}", vid, roomId);
        kick(staff, roomId, vid);

        return true;
    }

    public boolean kick(UserBase staff, long roomId, long vid) {
        Map<String, Object> requestBody = new HashMap<>();
        requestBody.put("uuid", staff.getUuid());
        requestBody.put("oprType", 1);
        requestBody.put("blacklist_vid",Arrays.asList(vid));
        String params = JSON.toJSONString(requestBody);
        LOGGER.info("kick, staff: {}, params: {}", staff, params);
        Optional<String> response = httpPoolClientDefault.postJson(qywxConfig.getDomain() + qywxConfig.getPath("kick-external"), params);
        if (response.isPresent()) {
            QwCommonResModel<RoomListResponse> roomInfo = QwCommonResModel.parseResponse(response.get(), RoomListResponse.class);
            if (roomInfo.getErrcode() == 0) {
                LOGGER.info("Kick external user {} from room {} successfully", vid, roomId);
                return true;
            } else {
                LOGGER.error("Failed to kick external user {} from room {}: {}", vid, roomId, roomInfo.getErrmsg());
            }
        } else {
            LOGGER.error("Failed to kick external user {} from room {}: No response", vid, roomId);
        }
        return false;
    }

    //增加校验，是否在群内
    public boolean isInRoom(UserBase staff, long roomId,long vid) {
        List<RoomMemberListInfo.Member> memberList = getRoomMemberList(staff, roomId);
        if (memberList == null || memberList.isEmpty()) {
            LOGGER.info("isInRoom, memberList is empty");
            return false;
        }
        LOGGER.info("isInRoom, memberList: {}", memberList);
        for (RoomMemberListInfo.Member member : memberList) {
            LOGGER.info("isInRoom, member: {}", member);
            if (member.getUin() == vid && member.getJoinScene() == 3) {
                return true;
            }
        }
        return false;
    }

    private List<RoomMemberListInfo.Member> getRoomMemberList(UserBase staff, long roomId) {
        Map<String, Object> requestBody = new HashMap<>();
        requestBody.put("uuid", staff.getUuid());
        requestBody.put("roomid", roomId);
        String params = JSON.toJSONString(requestBody);
        String url = qywxConfig.getDomain() + qywxConfig.getPath("get-roomMembers");
        Optional<String> response = httpPoolClientDefault.postJson(url, params);
        LOGGER.info("getRoomMemberList, params: {}, response: {},url: {}", params, response, url);
        List<RoomMemberListInfo.Member> memberList = new ArrayList<>();
        if (response.isPresent()) {
            QwCommonResModel<RoomMemberListInfo> memberInfo = QwCommonResModel.parseResponse(response.get(), RoomMemberListInfo.class);
            if (memberInfo.getErrcode() == 0) {
                memberList = memberInfo.getData().getMemberList();
            }
        }
        return memberList;
    }

    public void forceCloseRoomSwitch(String roomId) {
        WorkWechatRoomInfoExample example = new WorkWechatRoomInfoExample();
        example.createCriteria().andRoomIdEqualTo(roomId);
        List<WorkWechatRoomInfo> workWechatRoomInfos = workWechatRoomInfoMapper.selectByExample(example);
        if (CollectionUtils.isNotEmpty(workWechatRoomInfos)) {
            WorkWechatRoomInfo workWechatRoomInfo = workWechatRoomInfos.get(0);
            workWechatRoomInfo.setAutoRemoveUserSwitch(0);
            workWechatRoomInfo.setUpdateTime(new Date());
            workWechatRoomInfo.setShowTip(1);
            workWechatRoomInfo.setTipInfo("入群异常率过高，已关闭，群二维码务必只在票圈平台投放");
            workWechatRoomInfoMapper.updateByPrimaryKeySelective(workWechatRoomInfo);
            log.info("forceCloseRoomSwitch, roomId: {}, autoRemoveUserSwitch set to 0", roomId);
        }
    }
}
