package com.tzld.piaoquan.risk.control.service.rule;

import java.util.List;

/**
 * 解析
 *
 * @author supeng
 */
public class Parser {

    private final List<Token> tokens;
    private int current = 0;

    public Parser(List<Token> tokens) {
        this.tokens = tokens;
    }

    public Expression parse() {
        return parseExpression();
    }

    private Expression parseExpression() {
        Expression expr = parseTerm();
        while (match(TokenType.OR)) {
            expr = new BinaryOp(expr, parseTerm(), TokenType.OR);
        }
        return expr;
    }

    private Expression parseTerm() {
        Expression expr = parseFactor();
        while (match(TokenType.AND)) {
            expr = new BinaryOp(expr, parseFactor(), TokenType.AND);
        }
        return expr;
    }

    private Expression parseFactor() {
        if (match(TokenType.LPAREN)) {
            Expression expr = parseExpression();
            consume(TokenType.RPAREN, "Expect ')' after expression.");
            return new Parenthesis(expr);
        } else {
            return parseComparison();
        }
    }

    private Expression parseComparison() {
        Token leftToken = advance();
        String leftVar = null;
        Double leftNum = null;
        if (leftToken.type == TokenType.VARIABLE) {
            leftVar = leftToken.value;
        } else if (leftToken.type == TokenType.NUMBER) {
            leftNum = Double.parseDouble(leftToken.value);
        } else {
            throw new RuntimeException("Expected variable or number");
        }

        Token op = advance();
        if (!isComparisonOperator(op.type)) {
            throw new RuntimeException("Expected comparison operator");
        }

        Token rightToken = advance();
        String rightVar = null;
        Double rightNum = null;
        if (rightToken.type == TokenType.VARIABLE) {
            rightVar = rightToken.value;
        } else if (rightToken.type == TokenType.NUMBER) {
            rightNum = Double.parseDouble(rightToken.value);
        } else {
            throw new RuntimeException("Expected variable or number");
        }

        return new Comparison(leftVar, leftNum, op.type, rightVar, rightNum);
    }

    private boolean isComparisonOperator(TokenType type) {
        return type == TokenType.GT || type == TokenType.LT || type == TokenType.EQ
                || type == TokenType.GTE || type == TokenType.LTE || type == TokenType.NEQ;
    }

    private boolean match(TokenType type) {
        if (check(type)) {
            current++;
            return true;
        }
        return false;
    }

    private Token consume(TokenType type, String error) {
        if (check(type)) {
            return advance();
        }
        throw new RuntimeException(error);
    }

    private boolean check(TokenType type) {
        return current < tokens.size() && tokens.get(current).type == type;
    }

    private Token advance() {
        if (current < tokens.size()) {
            return tokens.get(current++);
        }
        // Return EOF
        return tokens.get(tokens.size() - 1);
    }
}
